//
//  AudioIO6AppDelegate.m
//  AudioIO
//
//  Created by Matthias Krauß on 02.10.13.
//  Copyright (c) 2013 Matthias Krauß. All rights reserved.
//

#import "AudioIO6AppDelegate.h"

#define BRIDGE 
#define SAMPLES_PER_PHASE 80
#define HALF_PHASE 40
#define MIN_PHASE 10
#define TRUE_HIGH_PHASE 60
#define FALSE_HIGH_PHASE 20
#define AVG_COUNT 10000
#define DELAY_SAMPLES 3
#define EDGE_THRES 0.5

typedef enum {
    STATE_IDLE = 0,
    STATE_START,
    STATE_BIT7,
    STATE_BIT6,
    STATE_BIT5,
    STATE_BIT4,
    STATE_BIT3,
    STATE_BIT2,
    STATE_BIT1,
    STATE_BIT0,
} TRANSFER_STATE;

@interface AudioIO6AppDelegate () 

@property (assign) AudioBufferList* inputBufferList;
@property (assign) AudioComponentInstance inputUnit;
@property (assign) Float64 inputSampleRate;
@property (assign) AudioComponentInstance outputUnit;

@property (retain) NSMutableArray* sendBuffer;

- (void) setupInput;
- (void) setupOutput;

- (void) parseBuffer:(float*)buffer numSamples:(UInt32)numSamples;
- (void) fillBuffer:(float*)buffer numSamples:(UInt32)numSamples;

- (void)parseBit:(BOOL)bit;
- (void)parseByte:(uint8_t)byte;


- (OSStatus) inputCallbackWithData:(AudioBufferList*)ioData
                         timeStamp:(const AudioTimeStamp*)inTimeStamp
                         busNumber:(UInt32)inBusNumber
                         numFrames:(UInt32)inNumberFrames
                       actionFlags:(AudioUnitRenderActionFlags*)ioActionFlags;
- (OSStatus) renderCallbackWithData:(AudioBufferList*)ioData
                          timeStamp:(const AudioTimeStamp*)inTimeStamp
                          busNumber:(UInt32)inBusNumber
                          numFrames:(UInt32)inNumberFrames
                        actionFlags:(AudioUnitRenderActionFlags*)ioActionFlags;

@end

OSStatus MyRenderCallback(void* inRefCon,
                          AudioUnitRenderActionFlags* ioActionFlags,
                          const AudioTimeStamp* inTimeStamp,
                          UInt32 inBusNumber,
                          UInt32 inNumberFrames,
                          AudioBufferList* ioData) {
    AudioIO6AppDelegate* obj = (BRIDGE AudioIO6AppDelegate*)inRefCon;
    return [obj renderCallbackWithData:ioData timeStamp:inTimeStamp busNumber:inBusNumber numFrames:inNumberFrames actionFlags:ioActionFlags];
}

OSStatus MyInputCallback(void* inRefCon,
                         AudioUnitRenderActionFlags* ioActionFlags,
                         const AudioTimeStamp* inTimeStamp,
                         UInt32 inBusNumber,
                         UInt32 inNumberFrames,
                         AudioBufferList* ioData) {
    AudioIO6AppDelegate* obj = (BRIDGE AudioIO6AppDelegate*)inRefCon;
    return [obj inputCallbackWithData:ioData timeStamp:inTimeStamp busNumber:inBusNumber numFrames:inNumberFrames actionFlags:ioActionFlags];
}

@implementation AudioIO6AppDelegate

@synthesize window = _window;
@synthesize sendField = _sendField;
@synthesize responseField = _responseField;

@synthesize inputBufferList = _inputBufferList;
@synthesize inputUnit = _inputUnit;
@synthesize inputSampleRate = _inputSampleRate;
@synthesize outputUnit = _outputUnit;
@synthesize sendBuffer = _sendBuffer;

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
    self.sendBuffer = [NSMutableArray arrayWithCapacity:100];
    [self setupInput];
    [self setupOutput];
}

- (void) setupInput {
    
    AudioComponentDescription inputDescription;
    inputDescription.componentType = kAudioUnitType_Output;
    inputDescription.componentSubType = kAudioUnitSubType_HALOutput;
    inputDescription.componentManufacturer = kAudioUnitManufacturer_Apple;
    inputDescription.componentFlags = 0;
    inputDescription.componentFlagsMask = 0;
    
    // Get components
    AudioComponent inputComponent = AudioComponentFindNext(NULL, &inputDescription);
    AudioComponentInstance inputUnit;
    OSStatus err = AudioComponentInstanceNew(inputComponent, &inputUnit);
    NSAssert(!err,@"AudioComponentInstanceNew failed");
    self.inputUnit = inputUnit;
    
    // Enable/disable inputs and outputs
    UInt32 on = 1;
    UInt32 off = 0;
    err = AudioUnitSetProperty(inputUnit,
                               kAudioOutputUnitProperty_EnableIO,
                               kAudioUnitScope_Input,
                               1,
                               &on,
                               sizeof(UInt32));
    NSAssert(!err,@"AudioUnitSetProperty failed input input");
    
    err = AudioUnitSetProperty(inputUnit,
                               kAudioOutputUnitProperty_EnableIO,
                               kAudioUnitScope_Output,
                               0,
                               &off,
                               sizeof(UInt32));
    NSAssert(!err,@"AudioUnitSetProperty failed input output");
    
    //Query default input device
    UInt32 size = sizeof(AudioDeviceID);
    AudioObjectPropertyAddress inPropertyAddress;
    inPropertyAddress.mSelector = kAudioHardwarePropertyDefaultInputDevice;
    inPropertyAddress.mScope = kAudioObjectPropertyScopeGlobal;
    inPropertyAddress.mElement = kAudioObjectPropertyElementMaster;
    AudioDeviceID inDeviceID;
    UInt32 propsize = sizeof(AudioDeviceID);
    err = AudioObjectGetPropertyData(kAudioObjectSystemObject, &inPropertyAddress, 0, NULL, &propsize, &inDeviceID);
    NSAssert(!err, @"AudioObjectGetPropertyData failed input");
    
    // Set the current device to the default input unit.
    err = AudioUnitSetProperty(inputUnit,
                               kAudioOutputUnitProperty_CurrentDevice,
                               kAudioUnitScope_Global,
                               0,
                               &inDeviceID,
                               sizeof(AudioDeviceID));
    NSAssert(!err, @"AudioUnitSetProperty failed - input device");
	
    //Query input sample rate
    AudioStreamBasicDescription inputDeviceFormat;
    UInt32 inputDeviceFormatSize = sizeof(AudioStreamBasicDescription);
    
    err = AudioUnitGetProperty (inputUnit,
								kAudioUnitProperty_StreamFormat,
								kAudioUnitScope_Input,
								1,
								&inputDeviceFormat,
								&inputDeviceFormatSize);
    NSAssert(!err, @"AudioUnitGetProperty failed - input native format");
    
    self.inputSampleRate = inputDeviceFormat.mSampleRate;
    NSLog(@"input sample rate is %f",self.inputSampleRate);
    
    //Set format
    AudioStreamBasicDescription asbd;
    asbd.mSampleRate = self.inputSampleRate;
    asbd.mFormatID = kAudioFormatLinearPCM;
    asbd.mFormatFlags = kAudioFormatFlagsNativeFloatPacked|kAudioFormatFlagIsNonInterleaved;
    asbd.mBytesPerPacket = sizeof(Float32);			//noninterleaved -> description for a single channel
    asbd.mFramesPerPacket = 1;
    asbd.mBytesPerFrame = sizeof(Float32); 			//noninterleaved -> description for a single channel
    asbd.mChannelsPerFrame = 1;						//noninterleaved -> number of buffer lists
    asbd.mBitsPerChannel = sizeof (Float32) * 8;
	asbd.mReserved=0;
    
    err = AudioUnitSetProperty(inputUnit,
                               kAudioUnitProperty_StreamFormat,
                               kAudioUnitScope_Output,
                               1,
                               &asbd,
                               sizeof(AudioStreamBasicDescription));
    NSAssert(!err, @"AudioUnitSetProperty failed - input asbd");
	
    UInt32 bufferSizeFrames = 0;
    size = sizeof(UInt32);
    err = AudioUnitGetProperty(inputUnit,
                               kAudioDevicePropertyBufferFrameSize,
                               kAudioUnitScope_Global,
                               0,
                               &bufferSizeFrames,
                               &size);
    NSAssert(!err, @"AudioUnitGetProperty failed - buffer size frames");
    
    
    UInt32 bufferSizeBytes = bufferSizeFrames * sizeof(Float32);
    
    // allocate an AudioBufferList plus enough space for array of AudioBuffers
    UInt32 bufferListSize = sizeof(AudioBufferList) + (sizeof(AudioBuffer) * (asbd.mChannelsPerFrame-1));
    
    //malloc buffer lists
    self.inputBufferList = (AudioBufferList*)malloc(bufferListSize);
    self.inputBufferList->mNumberBuffers = asbd.mChannelsPerFrame;
    
    //pre-malloc buffers for AudioBufferLists
    for(int i=0; i< self.inputBufferList->mNumberBuffers ; i++) {
        self.inputBufferList->mBuffers[i].mNumberChannels = 1;
        self.inputBufferList->mBuffers[i].mDataByteSize = bufferSizeBytes;
        self.inputBufferList->mBuffers[i].mData = malloc(bufferSizeBytes);
        memset(self.inputBufferList->mBuffers[i].mData, 0, bufferSizeBytes);
    }
	
    // Set input callback
    AURenderCallbackStruct inputCallbackStruct;
    inputCallbackStruct.inputProc = MyInputCallback;
    inputCallbackStruct.inputProcRefCon = (BRIDGE void*)(self);
    
    err = AudioUnitSetProperty(inputUnit,
                               kAudioOutputUnitProperty_SetInputCallback,
                               kAudioUnitScope_Global,
                               0,
                               &inputCallbackStruct,
                               sizeof(AURenderCallbackStruct));
    NSAssert(!err, @"AudioUnitSetProperty failed - input callback");
    
    err = AudioUnitInitialize(inputUnit);
    NSAssert(!err, @"AudioUnitInitialize failed - input");
    
    err = AudioOutputUnitStart(inputUnit);
    NSAssert(!err, @"AudioOutputUnitStart failed - input");
}

- (void) setupOutput {
    //Default Output-Komponente suchen
    AudioComponentDescription desc;
    desc.componentType = kAudioUnitType_Output;
    desc.componentSubType = kAudioUnitSubType_DefaultOutput;
    desc.componentManufacturer = kAudioUnitManufacturer_Apple;
    desc.componentFlags = 0;
    desc.componentFlagsMask = 0;
    
    AudioComponent comp = AudioComponentFindNext(NULL, &desc);
    NSAssert(comp, @"Could not find default out component");
    
    //Eine Instanz davon erzeugen
    AudioComponentInstance outputUnit;
    OSStatus err = AudioComponentInstanceNew(comp, &outputUnit);
    NSAssert1(!err,@"AudioComponentInstanceNew failed with error %i",err);
    self.outputUnit = outputUnit;
    
    //Audio-Stream-Format setzen
    AudioStreamBasicDescription asbd;
    asbd.mSampleRate = 44100.0;
    asbd.mFormatID = kAudioFormatLinearPCM;
    asbd.mFormatFlags = kAudioFormatFlagsNativeFloatPacked|kAudioFormatFlagIsNonInterleaved;
    asbd.mBytesPerPacket = sizeof(Float32);			//noninterleaved -> Beschreibung fuer EINEN Kanal
    asbd.mFramesPerPacket = 1;
    asbd.mBytesPerFrame = sizeof(Float32); 			//noninterleaved -> Beschreibung fuer EINEN Kanal
    asbd.mChannelsPerFrame = 1;						//noninterleaved -> Anzahl Bufferlisten
    asbd.mBitsPerChannel = sizeof (Float32) * 8;
	asbd.mReserved=0;
    
    err = AudioUnitSetProperty (outputUnit,
                                kAudioUnitProperty_StreamFormat,
                                kAudioUnitScope_Input,
                                0,
                                &asbd,
                                sizeof(asbd));
    NSAssert1(!err,@"AudioUnitSetProperty (asbd) failed with error %i",err);
    
    //Audio Unit initialisieren
    err = AudioUnitInitialize(outputUnit);
    NSAssert1(!err,@"AudioUnitInitialize failed with error %i",err);
    
    //Render Callback setzen
    AURenderCallbackStruct renderCallback;
    memset(&renderCallback, 0, sizeof(AURenderCallbackStruct));
    renderCallback.inputProc = MyRenderCallback;
    renderCallback.inputProcRefCon = (BRIDGE void*)self;
    err = AudioUnitSetProperty (outputUnit,
                                kAudioUnitProperty_SetRenderCallback,
                                kAudioUnitScope_Input,
                                0,
                                &renderCallback,
                                sizeof(AURenderCallbackStruct));
    NSAssert1(!err,@"AudioUnitSetProperty (callback) failed with error %i",err);
    
    //Los geht's!
    AudioOutputUnitStart(outputUnit);
}

- (OSStatus) inputCallbackWithData:(AudioBufferList*)ioData
                         timeStamp:(const AudioTimeStamp*)inTimeStamp
                         busNumber:(UInt32)inBusNumber
                         numFrames:(UInt32)inNumberFrames
                       actionFlags:(AudioUnitRenderActionFlags*)ioActionFlags {
    
    OSStatus err = AudioUnitRender(self.inputUnit,
                                   ioActionFlags,
                                   inTimeStamp,
                                   inBusNumber,     //will be '1' for input data
                                   inNumberFrames, //# of frames requested
                                   self.inputBufferList);
	NSAutoreleasePool* arp = [[NSAutoreleasePool alloc] init];
	float* samples = (float*)(self.inputBufferList->mBuffers[0].mData);
	[self parseBuffer:samples numSamples:inNumberFrames];
	[arp drain];
    return err;
}

- (OSStatus) renderCallbackWithData:(AudioBufferList*)ioData
                          timeStamp:(const AudioTimeStamp*)inTimeStamp
                          busNumber:(UInt32)inBusNumber
                          numFrames:(UInt32)inNumberFrames
                        actionFlags:(AudioUnitRenderActionFlags*)ioActionFlags {
	NSAutoreleasePool* arp = [[NSAutoreleasePool alloc] init];
	float* samples = (float*)(ioData->mBuffers[0].mData);
	[self fillBuffer:samples numSamples:inNumberFrames];
	[arp drain];
    return noErr;
}

- (void) parseBuffer:(float*)buffer numSamples:(UInt32)numSamples {
    static int avgCounter = 0;
    static float avg = 0.0f;
	static float maxDiff = 0.0f; 
	static float delayLine[DELAY_SAMPLES];
	static int delayLineIdx = 0;
    static BOOL wasHigh = NO;
    static UInt32 lastChangeCounter = 0;
	
    for (int i=0; i<numSamples; i++) {

        float sample = buffer[i];	//Sample lesen
		
		// Sample in Ringbuffer verzoegern
		delayLineIdx = (delayLineIdx + 1) % DELAY_SAMPLES;
		float delayedSample = delayLine[delayLineIdx];
		delayLine[delayLineIdx] = sample;
		
		//Aenderung (Flankensteilheit) berechnen
		float diff = sample - delayedSample;
		
		if (avgCounter <= AVG_COUNT) {	//Startup: Statistik ziehen
            avg += sample;
            avgCounter++;
			float absDiff = ABS(diff);
			if (absDiff > maxDiff) maxDiff = absDiff;
            if (avgCounter > AVG_COUNT) {	//Ende Startup
                avg /= avgCounter;
				NSLog(@"max diff is %f",maxDiff);
                NSLog(@"avg is %f",avg);
            }

        } else {	// Regulaeres Erkennen von Flanken
			
			lastChangeCounter++;	//Laenge des aktuellen Zustandes in Samples

			BOOL high = wasHigh;
			if ((lastChangeCounter > MIN_PHASE) && (diff > (EDGE_THRES*maxDiff))) high = NO;
			else if ((lastChangeCounter > MIN_PHASE) && (diff < (-EDGE_THRES*maxDiff))) high = YES;
			
            if (high != wasHigh) {	//Wechsel Low-High
                UInt32 duration = lastChangeCounter;
				NSLog(@"%i duration %i",wasHigh, duration);
                if (!high) {		//Wechsel High-Low
                    [self parseBit:(duration > HALF_PHASE) ? YES : NO];
                }
                lastChangeCounter = 0;
            }
			wasHigh = high;
        }
    }
}

- (void) fillBuffer:(float*)buffer numSamples:(UInt32)numSamples {
    static TRANSFER_STATE sendState = STATE_IDLE;
    static int sendPhase = 0;
    static uint8_t sendByte;
    static BOOL sendBit = NO;
	
    for (int i=0; i<numSamples; i++) {
        int pwmThreshold = sendBit ? TRUE_HIGH_PHASE : FALSE_HIGH_PHASE;
        float value = (sendPhase < pwmThreshold) ? -1.0f : 1.0f;
        buffer[i] = value;
        sendPhase++;
        if (sendPhase > SAMPLES_PER_PHASE) {
            sendPhase = 0;
            switch (sendState) {
                case STATE_IDLE:
                    if ([self.sendBuffer count] > 0) {
                        sendByte = [[self.sendBuffer lastObject] unsignedCharValue];
                        [self.sendBuffer removeLastObject];
                        sendState = STATE_START;
                        sendBit = 1;
                    }
                    break;
                case STATE_START:
                    sendBit = (sendByte & 0x80) ? TRUE : FALSE;
                    sendState = STATE_BIT7;
                    break;
                case STATE_BIT7:
                    sendBit = (sendByte & 0x40) ? TRUE : FALSE;
                    sendState = STATE_BIT6;
                    break;
                case STATE_BIT6:
                    sendBit = (sendByte & 0x20) ? TRUE : FALSE;
                    sendState = STATE_BIT5;
                    break;
                case STATE_BIT5:
                    sendBit = (sendByte & 0x10) ? TRUE : FALSE;
                    sendState = STATE_BIT4;
                    break;
                case STATE_BIT4:
                    sendBit = (sendByte & 0x08) ? TRUE : FALSE;
                    sendState = STATE_BIT3;
                    break;
                case STATE_BIT3:
                    sendBit = (sendByte & 0x04) ? TRUE : FALSE;
                    sendState = STATE_BIT2;
                    break;
                case STATE_BIT2:
                    sendBit = (sendByte & 0x02) ? TRUE : FALSE;
                    sendState = STATE_BIT1;
                    break;
                case STATE_BIT1:
                    sendBit = (sendByte & 0x01) ? TRUE : FALSE;
                    sendState = STATE_BIT0;
                    break;
                case STATE_BIT0:
                    sendBit = 0;
                    sendState = STATE_IDLE;
                    break;
                default:
                    NSAssert(false,@"Should not be here");
                    break;
            }
            
        }
    }
}

- (void)parseBit:(BOOL)bit {
    static TRANSFER_STATE receiveState = STATE_IDLE;
    static uint8_t receiveByte;
    switch (receiveState) {
        case STATE_IDLE:
            if (bit) {
                receiveByte = 0;
                receiveState = STATE_BIT7;
            }
            break;
        case STATE_BIT7:
            if (bit) receiveByte |= 0x80;
            receiveState = STATE_BIT6;
            break;
        case STATE_BIT6:
            if (bit) receiveByte |= 0x40;
            receiveState = STATE_BIT5;
            break;
        case STATE_BIT5:
            if (bit) receiveByte |= 0x20;
            receiveState = STATE_BIT4;
            break;
        case STATE_BIT4:
            if (bit) receiveByte |= 0x10;
            receiveState = STATE_BIT3;
            break;
        case STATE_BIT3:
            if (bit) receiveByte |= 0x08;
            receiveState = STATE_BIT2;
            break;
        case STATE_BIT2:
            if (bit) receiveByte |= 0x04;
            receiveState = STATE_BIT1;
            break;
        case STATE_BIT1:
            if (bit) receiveByte |= 0x02;
            receiveState = STATE_BIT0;
            break;
        case STATE_BIT0:
            if (bit) receiveByte |= 0x01;
            [self parseByte:receiveByte];
            receiveState = STATE_IDLE;
            break;
        default:
            break;
    }
}

- (void)parseByte:(uint8_t)byte {
//    NSLog(@"received byte %i",byte);
    if ((byte >= 32) && (byte < 128)) {
        NSString* responseText = [self.responseField stringValue];
        responseText = [responseText stringByAppendingFormat:@"%c",byte];
        if ([responseText length] > 30) responseText = [responseText substringFromIndex:[responseText length]-30];
        [self.responseField setStringValue:responseText];
    }
}

- (IBAction)sendText:(id)sender {
    NSString* string = [self.sendField stringValue];
    NSData* asciiData = [string dataUsingEncoding:NSASCIIStringEncoding allowLossyConversion:YES];
    NSUInteger len = [asciiData length];
    const uint8_t* ascii = (const uint8_t*)[asciiData bytes];
    for (int i=0; i<len; i++) {
        uint8_t byte = ascii[i];
        [self.sendBuffer insertObject:[NSNumber numberWithUnsignedChar:byte] atIndex:0];
    }
}


@end
