//
//  AppDelegate.m
//  BLE
//
//  Created by Matthias Krauß on 01.10.13.
//  Copyright (c) 2013 Matthias Krauß. All rights reserved.
//

#import "AppDelegate.h"

// TI Sensor Tag UUIDs (GATT):
// All UUIDs end with "-0451-4000-B000-000000000000"

// IR temp sens:
// Data: UUID F000AA01, handle 0x25, read/notify, 4 bytes: ObjLSB ObjMSB AmbLSB AmbMSB
// Data notif: UUID F000AA01, handle 0x26, readwrite, 2 bytes
// Config: UUID F000AA02, handle 0x2a, readwrite, 1 byte
// When enabled, ~1 measurement per sec
static NSString* IRTEMP_UUID        = @"F000AA00-0451-4000-B000-000000000000";
static NSString* IRTEMP_DATA_UUID   = @"F000AA01-0451-4000-B000-000000000000";
static NSString* IRTEMP_CONF_UUID   = @"F000AA02-0451-4000-B000-000000000000";

// Accelerometer:
// Data: UUID F000AA11, handle 0x2D, read/notify, 3 bytes: x y z (range: -2g..2g)
// Data notif: UUID F000AA11, handle 0x2E, read/write, 2 bytes
// Config: UUID F000AA12, handle 0x31, read/write, 1 byte
// Period: UUID F000AA13, handle 0x34, read/write, 1 byte: time (10ms units)
// Continuous measurement when enabled
static NSString* ACCEL_UUID         = @"F000AA10-0451-4000-B000-000000000000";
static NSString* ACCEL_DATA_UUID    = @"F000AA11-0451-4000-B000-000000000000";
static NSString* ACCEL_CONF_UUID    = @"F000AA12-0451-4000-B000-000000000000";
static NSString* ACCEL_PERIOD_UUID  = @"F000AA13-0451-4000-B000-000000000000";

// Humidity:
// Data: UUID F000AA21, handle 0x38, read/notify, 4 bytes: TempLSB TempMSB HumLSB HumMSB
// Data notif: UUID F000AA21, handle 0x39, read/write, 2 bytes
// Config: UUID F000AA22, handle 0x3C, read/write, 1 byte
// One-shot? Continuous?
static NSString* HUMID_UUID         = @"F000AA21-0451-4000-B000-000000000000";
static NSString* HUMID_DATA_UUID    = @"F000AA21-0451-4000-B000-000000000000";
static NSString* HUMID_CONF_UUID    = @"F000AA22-0451-4000-B000-000000000000";

// Magnetometer:
// Data: UUID F000AA31, handle 0x40, read/notify, 6 bytes: XLSB XMSB YLSB YMSB ZLSB ZMSB
// Data notif: UUID F000AA31, handle 0x41, read/write, 2 bytes
// Config: UUID F000AA32, handle 0x44, read/write 1 byte
// Period: UUID F000AA33, handle 0x47, read/write, 1 byte (10ms units)
static NSString* MAGNET_UUID        = @"F000AA30-0451-4000-B000-000000000000";
static NSString* MAGNET_DATA_UUID   = @"F000AA31-0451-4000-B000-000000000000";
static NSString* MAGNET_CONFIG_UUID = @"F000AA32-0451-4000-B000-000000000000";
static NSString* MAGNET_PERIOD_UUID = @"F000AA33-0451-4000-B000-000000000000";

// Barometer:
// Data: UUID F000AA41, handle 0x4B, readonly, 4 bytes: TempLSB TempMSB PressLSB PressMSB
// Data notif: UUID F000AA41, handle 0x4C, read/write 2 bytes
// Config: UUID F000AA42, handle 0x4F, read/write, 1 byte
// Calib: UUID F000AA43, handle 0x52, read/write, 16 bytes: C1LSB C1MSB…C8LSB C8MSB
// One measurement per enable
static NSString* BAROM_UUID         = @"F000AA40-0451-4000-B000-000000000000";
static NSString* BAROM_DATA_UUID    = @"F000AA41-0451-4000-B000-000000000000";
static NSString* BAROM_CONF_UUID    = @"F000AA42-0451-4000-B000-000000000000";
static NSString* BAROM_CALIB_UUID   = @"F000AA43-0451-4000-B000-000000000000";

// Gyro:
// Data: UUID F000AA51, handle 0x57, readonly, 6 bytes: XLSB XMSB YLSB YMSB ZLSB ZMSB
// Data notif: UUID F000AA51, handle 0x58, readwrite, 2 bytes
// Config: UUID F000AA52, handle 0x5B, readwrite, 1 byte (bitfield: 0 = off, 7= on, bit 0=x on, bit 1=y on, bit 2=z on)
// When enabled, continuous measurements
static NSString* GYRO_UUID          = @"F000AA50-0451-4000-B000-000000000000";
static NSString* GYRO_DATA_UUID     = @"F000AA51-0451-4000-B000-000000000000";
static NSString* GYRO_CONF_UUID     = @"F000AA52-0451-4000-B000-000000000000";

// Keys
// Data: UUID F000FFE1, handle 0x5F, readonly
// Buttons on device. Bit 0: left key, Bit 1: right key, Bit 2: side key
static NSString* BUTTONS_UUID       = @"F000AA50-0451-4000-B000-000000000000";
static NSString* BUTTONS_DATA_UUID  = @"F000AA52-0451-4000-B000-000000000000";




@interface AppDelegate ()

@property (strong) CBCentralManager* manager;
@property (strong) CBPeripheral* peripheral;
@property (strong) CBService* tempService;
@property (strong) CBCharacteristic* tempDataCharacteristic;
@property (strong) CBCharacteristic* tempConfigCharacteristic;
@property (strong) CBService* accelService;
@property (strong) CBCharacteristic* accelDataCharacteristic;
@property (strong) CBCharacteristic* accelConfigCharacteristic;
@property (strong) CBCharacteristic* accelPeriodCharacteristic;

- (void) checkStartScan;
- (void) startScan;

- (void) enableTempSensor:(BOOL)onOff;
- (void) enableAccelSensor:(BOOL)onOff;

- (CBCharacteristic*) findCharacteristicWithUUIDString:(NSString*)uuidString inService:(CBService*)service;

//Sensor utilities
- (float) objTempForRawObj:(int)rawObj ambTemp:(float)ambTemp;

//Logging utilities
- (void) dumpCBState;
- (NSString*) uuidToString:(CBUUID*)uuid;

@end

@implementation AppDelegate

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    self.manager = [[CBCentralManager alloc] initWithDelegate:self queue:nil];
    [self checkStartScan];
}

/********************************
 *** CBCentralManagerDelegate ***
 ********************************/

- (void)centralManagerDidUpdateState:(CBCentralManager *)central {
    [self checkStartScan];
}

- (void)centralManager:(CBCentralManager *)central didDiscoverPeripheral:(CBPeripheral *)peripheral advertisementData:(NSDictionary *)advertisementData RSSI:(NSNumber *)RSSI {
    NSString* name = [peripheral name];
    NSLog(@"didDiscoverPeripheral %@", name);
    if ([name isEqualToString:@"TI BLE Sensor Tag"]) {
        [self.manager stopScan];
        NSLog(@"trying to connect...");
        [self.statusField setStringValue:@"Connecting…"];
        peripheral.delegate = self;
        self.peripheral = peripheral;
        [self.manager connectPeripheral:peripheral options:nil];
    }
}

- (void)centralManager:(CBCentralManager *)central didConnectPeripheral:(CBPeripheral *)peripheral {
    NSLog(@"didConnectPeripheral %@",[peripheral name]);
    [self.statusField setStringValue:@"Discovering Services…"];
    [peripheral discoverServices:nil];
}

- (void)centralManager:(CBCentralManager *)central didFailToConnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error {
    [self.statusField setStringValue:@"Connection failed"];
    NSLog(@"didFailToConnectPeripheral %@",[peripheral name]);
}

- (void)centralManager:(CBCentralManager *)central didDisconnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error {
    [self.statusField setStringValue:@"Disconnected."];
    NSLog(@"didDisconnectPeripheral");
}

/****************************
 *** CBPeripheralDelegate ***
 ****************************/

-(void) peripheral:(CBPeripheral *)peripheral didDiscoverServices:(NSError *)error {
    if (error) {
        NSLog(@"didDiscoverServices failed: %@",error);
        [self.statusField setStringValue:@"Service discovery failed."];
    } else {
        NSLog(@"didDiscoverServices:");
        [self.statusField setStringValue:@"Discovering Service Characteristics…"];
        for (CBService* service in peripheral.services) {
            if ([service.UUID isEqual:[CBUUID UUIDWithString:IRTEMP_UUID]]) {
                self.tempService = service;
                [self.peripheral discoverCharacteristics:nil forService:self.tempService];
            }
            if ([service.UUID isEqual:[CBUUID UUIDWithString:ACCEL_UUID]]) {
                self.accelService = service;
                [self.peripheral discoverCharacteristics:nil forService:self.accelService];
            }
        }
    }
}

- (void)peripheral:(CBPeripheral *)peripheral didDiscoverCharacteristicsForService:(CBService *)service error:(NSError *)error {
    if (error) {
        NSLog(@"didDiscoverCharacteristicsForService failed: %@",error);
    } else {
        [self.statusField setStringValue:@"Starting services…"];
        if ([service isEqual:self.tempService]) {
            self.tempDataCharacteristic = [self findCharacteristicWithUUIDString:IRTEMP_DATA_UUID inService:service];
            self.tempConfigCharacteristic = [self findCharacteristicWithUUIDString:IRTEMP_CONF_UUID inService:service];
            [self enableTempSensor:YES];
        } else if ([service isEqual:self.accelService]) {
            self.accelDataCharacteristic = [self findCharacteristicWithUUIDString:ACCEL_DATA_UUID inService:service];
            self.accelConfigCharacteristic = [self findCharacteristicWithUUIDString:ACCEL_CONF_UUID inService:service];
            self.accelPeriodCharacteristic = [self findCharacteristicWithUUIDString:ACCEL_PERIOD_UUID inService:service];
            [self enableAccelSensor:YES];
        }
        
    }
};

- (void)peripheral:(CBPeripheral *)peripheral didUpdateValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error {
    [self.statusField setStringValue:@"Running."];
    if ([characteristic isEqual:self.tempDataCharacteristic]) [self updateTemp];
    if ([characteristic isEqual:self.accelDataCharacteristic]) [self updateAccel];
}


/***********************
 *** Private methods ***
 ***********************/

- (void) checkStartScan {
    CBCentralManagerState state = [self.manager state];
    if (state == CBCentralManagerStatePoweredOn) [self startScan];
}

- (void) startScan {
    [self.statusField setStringValue:@"Scanning for devices…"];
    [self.manager scanForPeripheralsWithServices:nil options:nil];
}

- (void) enableTempSensor:(BOOL)onOff {
    uint8_t onOffBuffer = onOff ? 1 : 0;
    NSData* onOffData = [NSData dataWithBytes:&onOffBuffer length:1];
    [self.peripheral writeValue:onOffData forCharacteristic:self.tempConfigCharacteristic type:CBCharacteristicWriteWithResponse];

    [self.peripheral setNotifyValue:onOff forCharacteristic:self.tempDataCharacteristic];
}

- (void) enableAccelSensor:(BOOL)onOff {
    uint8_t speedBuffer = 10;   //100ms
    NSData* speedData = [NSData dataWithBytes:&speedBuffer length:1];
    [self.peripheral writeValue:speedData forCharacteristic:self.accelPeriodCharacteristic type:CBCharacteristicWriteWithResponse];
    
    uint8_t onOffBuffer = onOff ? 1 : 0;
    NSData* onOffData = [NSData dataWithBytes:&onOffBuffer length:1];
    [self.peripheral writeValue:onOffData forCharacteristic:self.accelConfigCharacteristic type:CBCharacteristicWriteWithResponse];
    
    [self.peripheral setNotifyValue:onOff forCharacteristic:self.accelDataCharacteristic];
}

- (void) updateTemp {
    const uint8_t* data = [self.tempDataCharacteristic.value bytes];
    int objTemp = (data[1] << 8) | data[0];
    if (objTemp > 32767) objTemp -= 65536;
    int ambTemp = (data[3] << 8) | data[2];
    float ambDeg = ambTemp / 128.0f;
    float objDeg = [self objTempForRawObj:objTemp ambTemp:ambDeg];
    [self.objTempField setStringValue:[NSString stringWithFormat:@"%3.1f %C",objDeg,(unichar)0x2103]];
    [self.ambTempField setStringValue:[NSString stringWithFormat:@"%3.1f %C",ambDeg,(unichar)0x2103]];
}

- (float) objTempForRawObj:(int)rawObj ambTemp:(float)ambTemp {
    long double Vobj2 = (double)rawObj * .00000015625;
    long double Tdie2 = (double)ambTemp + 273.15;
    long double S0 = 6.4*pow(10,-14);
    long double a1 = 1.75*pow(10,-3);
    long double a2 = -1.678*pow(10,-5);
    long double b0 = -2.94*pow(10,-5);
    long double b1 = -5.7*pow(10,-7);
    long double b2 = 4.63*pow(10,-9);
    long double c2 = 13.4f;
    long double Tref = 298.15;
    long double S = S0*(1+a1*(Tdie2 - Tref)+a2*pow((Tdie2 - Tref),2));
    long double Vos = b0 + b1*(Tdie2 - Tref) + b2*pow((Tdie2 - Tref),2);
    long double fObj = (Vobj2 - Vos) + c2*pow((Vobj2 - Vos),2);
    long double Tobj = pow(pow(Tdie2,4) + (fObj/S),.25);
    Tobj = (Tobj - 273.15);
    return (float)Tobj;
}
- (void) updateAccel {
    const int8_t* data = [self.accelDataCharacteristic.value bytes];
    int8_t x = data[0];
    int8_t y = data[1];
    int8_t z = data[2];
    [self.xSlider setIntValue:x];
    [self.ySlider setIntValue:y];
    [self.zSlider setIntValue:z];
}

- (CBCharacteristic*) findCharacteristicWithUUIDString:(NSString*)uuidString inService:(CBService*)service {
    CBUUID* uuid = [CBUUID UUIDWithString:uuidString];
    for (CBCharacteristic* characteristic in service.characteristics) {
        if ([characteristic.UUID isEqual:uuid]) return characteristic;
    }
    return nil;
}

- (void) dumpCBState {
    NSString* state = nil;
    switch ([self.manager state]) {
        case CBCentralManagerStateUnsupported:
            state = @"This device does not support Bluetooth Low Energy.";
            break;
        case CBCentralManagerStateUnauthorized:
            state = @"This app is not authorized to use Bluetooth Low Energy.";
            break;
        case CBCentralManagerStatePoweredOff:
            state = @"Bluetooth on this device is currently powered off.";
            break;
        case CBCentralManagerStateResetting:
            state = @"The BLE Manager is resetting; a state update is pending.";
            break;
        case CBCentralManagerStatePoweredOn:
            state = @"Bluetooth LE is turned on and ready for communication.";
            break;
        case CBCentralManagerStateUnknown:
            state = @"The state of the BLE Manager is unknown.";
            break;
        default:
            state = @"The state of the BLE Manager is invalid.";
    }
    NSLog(@"State is %@",state);
}

- (NSString*) uuidToString:(CBUUID*)uuid {
    NSMutableString* ms = [NSMutableString stringWithString:@"0x"];
    NSData* data = [uuid data];
    NSUInteger len = [data length];
    const uint8_t* bytes = [data bytes];
    for (NSInteger i=0; i<len;i++) {
        [ms appendFormat:@"%02x",bytes[i]];
    }
    return ms;
}

@end
